==> sage-2005-03-03-0918-src-0.1.tar: sage-0.1/sage/rings/real_field.py <==
"""
The field of real numbers.
"""

##################################################################
# (Implementation not done.  Read about Decimal type
# here: http://www.python.org/dev/doc/devel/whatsnew/node8.html)
#  -- or -- use GMP?  MPFR ??
##################################################################

import gmpy

import ring
import real_number

class RealField(ring.Ring, object):
    """
    The field of real numbers.
    
    EXAMPLES:
        >>> R = RealField_mpf(); R
        Multi-precision Real Field
        >>> R('1/3')
        0.333333333333333333333
        
    Note that the second argument is the number of *bits* of precision,
    not the number of digits of precision:
        >>> R('1/3',100)
        0.3333333333333333333333333333333333333333
        >>> R('1/3',200)
        0.333333333333333333333333333333333333333333333333333333333333333333333
        
    If we create a real without quotes, we loose precision, because the real is
    turned into a Python float:
        >>> R(0.333333333333333333333)
        0.33333333333333331483
        
    We can also coerce rational numbers and integers into R, but
    coercing a polynomial in raising an exception.
        >>> Q = RationalField()
        >>> R(Q('1/3'))
        0.333333333333333333333
        >>> S = PolynomialRing(Q)
        >>> R(S.gen())
        Traceback (most recent call last):
        ...
        TypeError: unable to coerce x to a RealNumber
        >>> R.is_field()
        True
        >>> R.characteristic()
        0
        >>> R.name()
        'R'
        >>> R == R
        True
        >>> R == 5
        False
    """
    def __new__(cls, *args, **kwds):
        if cls != RealField:
            return object.__new__(cls, *args ,**kwds)
        return RealField_mpf(*args, **kwds)

    def __call__(self, x, prec=0):
        raise NotImplementedError

    def __cmp__(self, other):
        raise NotImplementedError

    def __repr__(self):
        return "Real Field"

    def characteristic(self):
        """
        Returns the characteristic of the real field, which is 0.
        """
        return 0
        
    def is_field(self):
        """
        Returns True, since the real field is a field.
        """
        return True

    def name(self):
        """
        Returns a short string 'R' that describes the real field.
        """
        return "R"
        
    def pi(self, n=0):
        """
        Returns the constant pi=3.1415... with at least n bits of precision.
        
        EXAMPLES:
            >>> RealField_mpf().pi(128)
            3.141592653589793238462643383279502884197
        """
        return self(gmpy.pi(n))

_obj_RR_decimal = {}
class _uniq_RR_decimal(object):
    def __new__(cls):
        if _obj_RR_decimal.has_key(0):
            return _obj_RR_decimal[0]
        O = object.__new__(cls)
        _obj_RR_decimal[0] = O
        return O
        
class RealField_decimal(RealField, _uniq_RR_decimal):
    """
    EXAMPLES:
        >>> R = RealField_decimal(); S = RealField_mpf()
        >>> R(1) + S(1)
        2.0
        >>> S(1.1) - R(1.2)
        -0.0999999999999999111822
        >>> R.pi() * S.pi()
        9.869604401089358618817882133
        >>> R(2)*S(2)
        4.0
        >>> S(2) / R(3)
        0.666666666666666666667
        >>> R(2) < S(3)
        True
        >>> S(2.99) > R(3)
        False
    """
    def __call__(self, x):
        return real_number.RealNumber_decimal(x)

    def __cmp__(self, other):
        if isinstance(other, RealField_decimal):
            return 0
        return -1
    
    def __repr__(self):
        return "Infinite Precision Real Field"


_obj_RR_mpf = {}
class _uniq_RR_mpf(object):
    def __new__(cls):
        if _obj_RR_mpf.has_key(0):
            return _obj_RR_mpf[0]
        O = object.__new__(cls)
        _obj_RR_mpf[0] = O
        return O
        
class RealField_mpf(RealField, _uniq_RR_mpf):
    """
    EXAMPLES:
        >>> R = RealField_decimal(); S = RealField_mpf()
        >>> R(1) + S(1)
        2.0
        >>> R(2)*S(2)
        4.0
    """
    def __call__(self, x, prec=0):
        return real_number.RealNumber_mpf(x,prec)

    def __cmp__(self, other):
        if isinstance(other, RealField_mpf):
            return 0
        return -1
        
    def __repr__(self):
        return "Multi-precision Real Field"

RR_mpf = RealField_mpf()
RR_decimal = RealField_decimal()

# doctests
if __name__ ==  '__main__':
    import doctest, sys
    from rational_field import RationalField
    from polynomial_ring import PolynomialRing
    doctest.testmod(sys.modules[__name__])

==> sage-2005-03-04-1946-src-0.1.tar: sage-0.1/sage/rings/real_field.py <==
"""
The field of real numbers.
"""

##################################################################
# (Implementation not done.  Read about Decimal type
# here: http://www.python.org/dev/doc/devel/whatsnew/node8.html)
#  -- or -- use GMP?  MPFR ??
##################################################################

import gmpy

import ring
import sage.rings.real_number

class RealField(ring.Ring, object):
    """
    The field of real numbers.
    
    EXAMPLES:
        >>> R = RealField_mpf(); R
        Multi-precision Real Field
        >>> R('1/3')
        0.333333333333333333333
        
    Note that the second argument is the number of *bits* of precision,
    not the number of digits of precision:
        >>> R('1/3',100)
        0.3333333333333333333333333333333333333333
        >>> R('1/3',200)
        0.333333333333333333333333333333333333333333333333333333333333333333333
        
    If we create a real without quotes, we loose precision, because the real is
    turned into a Python float:
        >>> R(0.333333333333333333333)
        0.33333333333333331483
        
    We can also coerce rational numbers and integers into R, but
    coercing a polynomial in raising an exception.
        >>> Q = RationalField()
        >>> R(Q('1/3'))
        0.333333333333333333333
        >>> S = PolynomialRing(Q)
        >>> R(S.gen())
        Traceback (most recent call last):
        ...
        TypeError: unable to coerce x to a RealNumber
        >>> R.is_field()
        True
        >>> R.characteristic()
        0
        >>> R.name()
        'R'
        >>> R == R
        True
        >>> R == 5
        False
    """
    def __new__(cls, *args, **kwds):
        if cls != RealField:
            return object.__new__(cls, *args ,**kwds)
        return RealField_mpf(*args, **kwds)

    def __call__(self, x, prec=0):
        raise NotImplementedError

    def __cmp__(self, other):
        raise NotImplementedError

    def __repr__(self):
        return "Real Field"

    def characteristic(self):
        """
        Returns the characteristic of the real field, which is 0.
        """
        return 0
        
    def is_field(self):
        """
        Returns True, since the real field is a field.
        """
        return True

    def name(self):
        """
        Returns a short string 'R' that describes the real field.
        """
        return "R"
        
    def pi(self, n=0):
        """
        Returns the constant pi=3.1415... with at least n bits of precision.
        
        EXAMPLES:
            >>> RealField_mpf().pi(128)
            3.141592653589793238462643383279502884197
        """
        return self(gmpy.pi(n))

_obj_RR_decimal = {}
class _uniq_RR_decimal(object):
    def __new__(cls):
        if _obj_RR_decimal.has_key(0):
            return _obj_RR_decimal[0]
        O = object.__new__(cls)
        _obj_RR_decimal[0] = O
        return O
        
class RealField_decimal(RealField, _uniq_RR_decimal):
    """
    EXAMPLES:
        >>> R = RealField_decimal(); S = RealField_mpf()
        >>> R(1) + S(1)
        2.0
        >>> S(1.1) - R(1.2)
        -0.0999999999999999111822
        >>> R.pi() * S.pi()
        9.869604401089358618817882133
        >>> R(2)*S(2)
        4.0
        >>> S(2) / R(3)
        0.666666666666666666667
        >>> R(2) < S(3)
        True
        >>> S(2.99) > R(3)
        False
    """
    def __call__(self, x):
        return sage.rings.real_number.RealNumber_decimal(x)

    def __cmp__(self, other):
        if isinstance(other, RealField_decimal):
            return 0
        return -1
    
    def __repr__(self):
        return "Infinite Precision Real Field"


_obj_RR_mpf = {}
class _uniq_RR_mpf(object):
    def __new__(cls):
        if _obj_RR_mpf.has_key(0):
            return _obj_RR_mpf[0]
        O = object.__new__(cls)
        _obj_RR_mpf[0] = O
        return O
        
class RealField_mpf(RealField, _uniq_RR_mpf):
    """
    EXAMPLES:
        >>> R = RealField_decimal(); S = RealField_mpf()
        >>> R(1) + S(1)
        2.0
        >>> R(2)*S(2)
        4.0
    """
    def __call__(self, x, prec=0):
        return sage.rings.real_number.RealNumber_mpf(x,prec)

    def __cmp__(self, other):
        if isinstance(other, RealField_mpf):
            return 0
        return -1
        
    def __repr__(self):
        return "Multi-precision Real Field"

RR_mpf = RealField_mpf()
RR_decimal = RealField_decimal()

# doctests
if __name__ ==  '__main__':
    import doctest, sys
    from rational_field import RationalField
    from polynomial_ring import PolynomialRing
    doctest.testmod(sys.modules[__name__])
